<?php
/*
Plugin Name: Delete Duplicate Posts
Plugin Script: delete-duplicate-posts.php
Plugin URI: http://www.myWordPress.com
Description: View and delete those pesky Duplicate Posts on your WordPress blog!
Version: 1.3.1
Author: myWordPress.com
Author URI: http://www.myWordPress.com
Min WP Version: 2.7
Max WP Version: 2.8.4
Update Server: http://www.myWordPress.com

== Changelog ==

= 1.3.1 =
* Fixes problem with dashboard widget. Thanks to Derek for pinpointing the error.

= 1.3 =
* Ensures all post meta for the deleted blogposts are also removed...

= 1.1 =
* Uses internal delete function, which also cleans up leftover meta-data. Takes a lot more time to complete however and might time out on some hosts.

= 1.0 =
* First release

*/


// ### Adding the admin menu to the WP-Admin interface
function deleteduplicateposts_admin_menu() {
  if (function_exists('add_submenu_page')) {
    add_management_page('Duplicate Posts', 'Delete Duplicate Posts', 8, basename(__FILE__), 'deleteduplicateposts_admin_options');


  }
}

// ### The Admin options/settings screen
function deleteduplicateposts_admin_options(){
 	global $wpdb, $wp_version,$wp_locale,$current_blog,$wp_rewrite;
 	$table_name = $wpdb->prefix . "posts";	
	$blog_id = $current_blog->blog_id;
	$baseurl=WP_CONTENT_URL . '/plugins/' . plugin_basename(dirname(__FILE__)). '/';	
	$pluginfo=get_plugin_data(__FILE__);
	$version=$pluginfo['Version'];	
	$limit=50; // Todo: make this userdefinable

//----------------------------------------------
// DELETE DUPLICATE POSTS!!
//----------------------------------------------

	if ( ($_POST['action'] == 'delete') && $_POST['deleteduplicateposts_delete'] ) {
	
		
		$nonce=$_POST['_deleteduplicatepostsnonce'];
		
	    if (! wp_verify_nonce($nonce, 'deleteduplicateposts') ) die("Failed security check");
		if ( function_exists('current_user_can') && current_user_can('edit_plugins')) {

			$query="select bad_rows.*
			from $table_name as bad_rows
			inner join (
			select post_title,id, MIN(id) as min_id
			from $table_name 
			WHERE (
			(
			`post_status` = 'published'
			)
			OR (
			`post_status` = 'publish'
			)
			)
			group by post_title
			having count(*) > 1
			) as good_rows on good_rows.post_title = bad_rows.post_title
			and good_rows.min_id <> bad_rows.id;";
			$dupes = $wpdb->get_results($query);
		
			$resultnote='';
			$dispcount=0;
			foreach ($dupes as $dupe) {
				
				
				$postid=$dupe->ID;
				$title=$dupe->post_title;
				$perma=get_permalink($postid);
				
				if ($postid<>''){

					$custom_field_keys = get_post_custom_keys($postid);
					foreach ( $custom_field_keys as $key => $value ) {
				//		$valuet = trim($value);
				//		if ( '_' == $valuet{0} )
				//		continue;
						delete_post_meta($postid, $key, '');
						$resultnote .= "-- Post META key '". $key . "' value ('$value') deleted! <br />";
					}
					
					$result = wp_delete_post($postid);
					if (!$result) {	
						$resultnote .= "!! Problem deleting post $postid - $perma !!<br>";
					}
					else {	
						$resultnote .= "'$title' (ID #$postid) Deleted!<br>";


						$dispcount++;
					}
				}

			}			
			
			$resultnote .="Total: <strong>$dispcount</strong> deleted duplicate posts!";
			if ($resultnote<>'') echo '<div class="updated fade"><p>' . $resultnote. '</p></div>';
		}

	}
		  

//----------------------------------------------
// FIND ALL DUPLICATES!!
//----------------------------------------------


	if ( ($_POST['action'] == 'research') && $_POST['deleteduplicateposts_research'] ) {
	
		
		$nonce=$_POST['_deleteduplicatepostsnonce'];
			
		$myvideoposter_postvideonow=$_POST['myvideoposter_postvideonow'];
		
	    if (! wp_verify_nonce($nonce, 'deleteduplicateposts') ) die("Failed security check");
		if ( function_exists('current_user_can') && current_user_can('edit_plugins')) {

			$search4dupes=true;

		//	if ($postnowresult<>'') echo '<div class="updated fade"><p>' . $postnowresult. '</p></div>';
		}

	}
		 
		 
		 
    ?>
<div class="wrap">
	

	<h2><?php _e("Delete Duplicate Posts v.$version", 'myvideoposter'); ?> </h2>
    <br class="clear" />
    <p><em>- A clever WordPress Plugin by <a href="http://mywordpress.com">myWordPress.com</a></em></p>




<div class="tool-box">
	<div id="count">
		<h3 class="title">Check For Duplicate Posts</h3>
		<p>Clicking the button finds duplicated posts. You can then delete them individually, or all with one click!</p>
		<p>After searching, it lists all posts, and beneath the list, you can click the button to delete them all.</p>
		<div class="buttons">
			<form name="form" method="post" action="<?php echo $location; ?>">
			<p class="submit">
			<?php $nonce= wp_create_nonce('deleteduplicateposts');?>					
			<input type="hidden" name="_deleteduplicatepostsnonce" value="<?php echo $nonce; ?>" />
			<input type="hidden" name="action" value="research" />
				
			<input class="button-primary" type="submit" name="deleteduplicateposts_research" value="<?php _e('Search for Duplicate Posts'); ?>"
								<br />
								

			</p>	
			</form>
		</div>
		
		<?php
		
		if ($search4dupes) {
		
			$query="select bad_rows.*
			from $table_name as bad_rows
			inner join (
			select post_title,id, MIN(id) as min_id
			from $table_name 
			WHERE (
			(
			`post_status` = 'published'
			)
			OR (
			`post_status` = 'publish'
			)
			)
			group by post_title
			having count(*) > 1
			) as good_rows on good_rows.post_title = bad_rows.post_title
			and good_rows.min_id <> bad_rows.id;";
			$dupes = $wpdb->get_results($query);
			 
			$dupescount = count($dupes); 
			$limit=1000000; // REALLY high limit... for V2 ..
			if ($dupescount<>'') {
				if ($dupescount>$limit) {
					echo "<h4><strong>$dupescount</strong> dupes found! ($limit is displayed)</h4>";
				}
				else {
					echo "<h4><strong>$dupescount</strong> dupes found!</h4>";
				}
				$plugurl=WP_CONTENT_URL . '/plugins/' .plugin_basename(__FILE__) ;	
				echo '<table class="widefat post" cellspacing="0">';
			
				echo '<tr><td><strong>Post title</td><td><td><strong>Direct link</strong></td><strong>Post ID</strong></td><td></tr>';
			
			//	echo '<tr><td><strong>Post title</td><td><td><strong>Direct link</strong></td><strong>Post ID</strong></td><td><strong>Delete post?</strong></td></tr>'; //V2 table!
			
				$dispcount=0;
				foreach ($dupes as $dupe) {
					$dispcount++;
					$postid=$dupe->ID;
					//$deleteurl = "tools.php?page=delete-duplicate-posts.php?nonce=$nonce&delid=$postid&del=true"; //V2
			
					echo "<tr><td>".$dupe->post_title ."</td><td><td><a href='".get_permalink($postid)."' target='_blank'>View</a></td></tr>";
					
					//if ($dispcount>$limit) break; //V2 ... limit listing
				}
				echo "</table>";
			?>	
			<div class="buttons">
			<form name="form2" method="post" action="<?php echo $location; ?>">
			<p class="submit">
			<?php $nonce= wp_create_nonce('deleteduplicateposts');?>					
			<input type="hidden" name="_deleteduplicatepostsnonce" value="<?php echo $nonce; ?>" />
			<input type="hidden" name="action" value="delete" />
				
			<input class="button-primary" type="submit" name="deleteduplicateposts_delete" value="<?php _e('Delete listed duplicate posts!'); ?>"
								<br />
								

			</p>	
			</form>
			<p><strong>WARNING:</strong>If you click this button, the posts listed above <em> WILL BE PERMANENTLY ERASED</em> There is NO undo. Use at your own risk!</p>
		<?php	
			}
			else
			{
				echo "<h4>Congratulations! No duplicate posts detected!</h4>";
			}
			}
		?>
	

		</div>	
		
	</div>

</div>


<?php





?>
    
		
		<div id="poststuff" class="ui-sortable">
			<div class="postbox open" >
				<h3><?php _e('Plugin Credits', 'deleteduplicateposts') ?></h3>
				<div class="inside">
				<?php
					$baseurl=WP_CONTENT_URL . '/plugins/' . plugin_basename(dirname(__FILE__)). '/mywordpresssm.jpg';	
					?>
					<p><?php _e('This Plugin was created by', 'myvideoposter'); ?></p>
					<p><?php _e("<a href='http://mywordpress.com' target='_blank'><img src='$baseurl' border=0></a>", 'myvideoposter'); ?></p>

					
				</div>
			</div>
		</div>

   		<script type="text/javascript">
		<!--
		<?php if ( version_compare( $wp_version, '2.6.999', '<' ) ) { ?>
		jQuery('.postbox h3').prepend('<a class="togbox">+</a> ');
		<?php } ?>
		jQuery('.postbox h3').click( function() { jQuery(jQuery(this).parent().get(0)).toggleClass('closed'); } );
		jQuery('.postbox.close-me').each(function(){
			jQuery(this).addClass("closed");
		});
		//-->
		</script>
</div> 

<?php
 
}


if (!function_exists('seobooster_dashboard_widget_function'))
{
function seobooster_dashboard_widget_function() {
	include_once(ABSPATH . WPINC . '/rss.php');
	$baseurl=WP_CONTENT_URL . '/plugins/' . plugin_basename(dirname(__FILE__)). '/mywordpresssm.jpg';
	echo '<div class="rss-widget">';	
	echo "<p><a href='http://mywordpress.com' target='_blank'><img src='$baseurl' class='alignright' border=0 ></a>";	
	$rss = fetch_rss('http://feeds2.feedburner.com/mywordpresscom');
	
	if ($rss) {
	    $items = @array_slice($rss->items, 0, 3);

	    if (empty($items)) 
	    	echo '<li>No news right now.</li>';
	    else {
	    	foreach ( $items as $item ) { ?>
	    	<?php
	 //   	var_dump($item);
	    	?>
	    	<a href='<?php echo $item['link']; ?>' class="rsswidget" ><?php echo $item['title']; ?></a>

			<?php	
			$summary=$item['summary'];
			$place = strpos($summary,'Post from: <a href="http://mywordpress.com">WordPress Plugins - myWordPress</a>');
			$summary = str_replace('Post from: <a href="http://mywordpress.com">WordPress Plugins - myWordPress</a>', '',$summary);
	    		 
	    		 
	 ?><div class="rssSummary"> <?php 

	echo substr($summary,$place);	 
	 
	  ?></div>

	    	<?php }
	    }
	}
	?>
	</div>
	<?php

} 

} 

if (!function_exists('seobooster_add_dashboard_widgets'))
{
// Create the function use in the action hook
function seobooster_add_dashboard_widgets() {

	global $wp_version;

	if (version_compare($wp_version,"2.6","<")){
		exit ('');
	} 
	else
	{
		wp_add_dashboard_widget('seobooster_dashboard_widget', 'Clever WordPress Plugins', 'seobooster_dashboard_widget_function');	
		}
} 
} 


add_action('wp_dashboard_setup', 'seobooster_add_dashboard_widgets' ); // Lets add the Dashboard Widget..

add_action('admin_menu', 'deleteduplicateposts_admin_menu');