<?php
/*
Plugin Name: Tidy Up
Plugin URI: http://www.urbangiraffe.com/plugins/tidy-up
Description: Runs an HTML Tidy on all posts and comments.  See Manage screen for usage.
Author: John Godley
Version: 1.3
Author URI: http://www.urbangiraffe.com/
*/

define ('TIDY_ADMIN_LEVEL', 'edit_plugins');

if (is_admin ())
{
	function tidy_up_admin ()
	{
		if (isset ($_GET['post']) && current_user_can (TIDY_ADMIN_LEVEL))
			tidy_show_post (intval ($_GET['post']));
		else
			tidy_show_all ();
	}
	
	function tidy_show_post ($post)
	{
		include ('tidy_class.php');

		$save = false;
		if (isset ($_POST['clean']) && current_user_can (TIDY_ADMIN_LEVEL))
		{
			$input  = $_POST['input'];
			$output = $_POST['output'];

			update_option ('tidy_input', $input);
			update_option ('tidy_output', $output);
			$save = true;
		}

		$input = get_option ('tidy_input');
		$output = get_option ('tidy_output');

		if (!$input)
			$input = 'wordpress';

		if (!$output)
			$output = 'wordpress';

		$returnpage = get_bloginfo ('wpurl').'/wp-admin/edit.php';
		if ($save)
		{
			?>
			<div class="fade updated" id="message">
			  <p>Post has been cleaned and updated</p>
			</div>
			<?php
		}
		?>
		<div class="wrap">
		  <h2>Report for post <?php echo $post ?></h2>
		
		<?php if ($post > 0) : ?>
			<ul class="report"><?php tidy_post ($post, $save, $input, $output); ?>
		<?php else : ?>
			<p>Invalid post</p>
		<?php endif; ?>
			<a href="<?php echo $returnpage ?>">Return to manage page</a>
		</div>
		
		<div class="wrap">
		  <h2>Clean and update post <?php echo $post ?></h2>
			<p>Note that use of this option can seriously affect the health of your blog.  Please be aware of what you are doing, and <strong>backup your database</strong> before saving.  Absolutley no liability is taken for any damage caused.</strong></p>
			<ul>
			  <li>Default WordPress - Let the default WordPress auto-formatting routines handle the data.</li>
			  <li>Raw XHTML - Perform no additional WordPress auto-formatting.</li>
			</ul>
			
			<form method="post" action="<?php echo $_SERVER['REQUEST_URI']?>">
			<table>
				<tr>
					<td>Input:</td>
					<td>
						<select name="input">
					  	<option value="wordpress"<?php if (get_option ('tidy_input') == 'wordpress') :?>selected="selected"<?php endif; ?>>Default WordPress</option>
					    <option value="xhtml"<?php if (get_option ('tidy_input') == 'xhtml') :?>selected="selected"<?php endif; ?>>Raw XHTML</option>
					  </select> Output:   <select name="output">
						    <option value="wordpress"<?php if (get_option ('tidy_output') == 'wordpress') :?>selected="selected"<?php endif; ?>>Default WordPress</option>
						    <option value="xhtml"<?php if (get_option ('tidy_output') == 'xhtml') :?>selected="selected"<?php endif; ?>>Raw XHTML</option>
						  </select>
					</td>
				</tr>
				<tr>
				  <td></td>
					<td><input type="submit" name="clean" value="<?php echo __('Clean', 'tidy_up')?> &raquo;" /></td>
				</tr>
			</table>

			</form>
		</div>
		<?php
	}
	
	function tidy_show_all ()
	{
		if (current_user_can (TIDY_ADMIN_LEVEL))
		{
			$input  = $_POST['input'];
			$output = $_POST['output'];

			update_option ('tidy_input', $input);
			update_option ('tidy_output', $output);
		}
	?>
	
	<div class="wrap">
	  <h2>Tidy Up</h2>
		<p>Tidy Up will run <a href="http://tidy.sourceforge.net/">HTML Tidy</a> through all your posts or comments and produce a report.  You can use this to clean up your HTML, or you can have the plugin automatically update the data with cleaned XHTML from Tidy.</p>
		<p>NOTE: Cleaning your data can seriously affect the health of your blog.  Be aware of what you are doing and <strong>please do backup your database</strong>.  Absolutley no liability is taken for any damage caused.</p>

		<form method="post" action="<?php echo $_SERVER['REQUEST_URI']?>">
		<table>
		  <tr>
		    <td>Source:</td>
		    <td>
					<select name="type">
			     <option value="posts"<?php if ($_POST['type'] == 'posts') echo ' selected="selected"' ?>>Posts</option>
			     <option value="pages"<?php if ($_POST['type'] == 'pages') echo ' selected="selected"' ?>>Pages</option>
			     <option value="comments"<?php if ($_POST['type'] == 'comments') echo ' selected="selected"' ?>>Comments</option>
			     <option value="excerpts"<?php if ($_POST['type'] == 'excerpts') echo ' selected="selected"' ?>>Excerpts</option>
				 	</select>
				</td>
			</tr>
			<tr>
				<td>Input:</td>
				<td>
					<select name="input">
				  	<option value="wordpress"<?php if (get_option ('tidy_input') == 'wordpress') :?>selected="selected"<?php endif; ?>>Default WordPress</option>
				    <option value="xhtml"<?php if (get_option ('tidy_input') == 'xhtml') :?>selected="selected"<?php endif; ?>>Raw XHTML</option>
				  </select> Output:   <select name="output">
					    <option value="wordpress"<?php if (get_option ('tidy_output') == 'wordpress') :?>selected="selected"<?php endif; ?>>Default WordPress</option>
					    <option value="xhtml"<?php if (get_option ('tidy_output') == 'xhtml') :?>selected="selected"<?php endif; ?>>Raw XHTML</option>
					  </select>
					
					Offset: <input type="text" name="offset" value="<?php echo $_POST['offset'] ?>" size="5"/>
					Maximum items: <input type="text" name="maximum" value="<?php echo $_POST['maximum'] > 0 ? $_POST['maximum'] : 1000 ?>" size="5"/>
				</td>
			</tr>
			<tr>
			  <td></td>
				<td><input class="goodbutton" type="submit" name="report" value="Report"/> <input type="submit" name="clean" value="<?php echo __('Clean', 'tidy_up')?>" class="deathbutton" /></td>
			</tr>
		</table>
		</form>
	</div>
	
	<?php
		include ('tidy_class.php');
		
		$input = get_option ('tidy_input');
		$output = get_option ('tidy_output');
		if (!$input) $input = 'wordpress';
		if (!$output) $output = 'wordpress';
		
		if (current_user_can (TIDY_ADMIN_LEVEL) && (isset ($_POST['report']) || isset ($_POST['clean'])))
		{
			$offset = intval ($_POST['offset']);
			$max    = intval ($_POST['maximum']);
			
			if (!ini_get ('safe_mode'))
				set_time_limit (0);
			?>
			<div class="wrap">
			<h2>Report</h2>
			
			<ul class="report">
			<?php if ($_POST['type'] == "posts")
			  			tidy_run_posts (isset ($_POST['clean']) ? true : false, $input, $output, $offset, $max);
						else if ($_POST['type'] == 'pages')
			  			tidy_run_pages (isset ($_POST['clean']) ? true : false, $input, $output, $offset, $max);
			  		else if ($_POST['type'] == 'excerpts')
			  			tidy_run_excerpts (isset ($_POST['clean']) ? true : false, $input, $output, $offset, $max);
			 			else
							tidy_run_comments (isset ($_POST['clean']) ? true : false, $input, $output, $offset, $max);
			?>
			</ul>
			</div>
			<?php
		}
	}
	
	function tidy_up_menu ()
	{
    	add_management_page ("Tidy Up", "Tidy Up", TIDY_ADMIN_LEVEL, basename (__FILE__), "tidy_up_admin");
	}
	
	function tidy_up_head ()
	{
	?>
	<link rel="stylesheet" href="<?php echo get_bloginfo ('wpurl').'/wp-content/plugins/tidy-up/tidy_up.css' ?>" type="text/css"/>
	<script type="text/javascript" language="javascript" charset="utf-8">
	// <![CDATA[
		var wp_base = '<?php bloginfo ('wpurl') ?>/wp-content/plugins/tidy-up/ajax.php';
		var wp_loading = '<?php _e ('Loading...', 'tidy-up') ?>';
	// ]]>
	</script>
	<script type="text/javascript" src="<?php bloginfo ('wpurl') ?>/wp-content/plugins/tidy-up/prototype.js"></script>
	<script type="text/javascript" src="<?php bloginfo ('wpurl') ?>/wp-content/plugins/tidy-up/tidy_up.js"></script>
	<?php
	}
	
	function tidy_menu_column ($name, $id = '')
	{
		if (current_user_can (TIDY_ADMIN_LEVEL)) {
		?>
		<a href="edit.php?page=tidy_up.php&amp;post=<?php echo $id ?>">Tidy</a>
		<?php }
	}
	
	function tidy_menu_add ($columns)
	{
		if (current_user_can (TIDY_ADMIN_LEVEL))
			$columns['tidy_up'] = '';
		return $columns; 
	}
	
	add_filter ('admin_menu',                 'tidy_up_menu');
	add_action ('manage_posts_custom_column', 'tidy_menu_column', 10, 2);
	add_filter ('manage_posts_columns',       'tidy_menu_add');

	if (strstr ($_SERVER['REQUEST_URI'], 'tidy_up'))
		add_filter ('admin_head', 'tidy_up_head');
}

?>